<?php

require_once("config.inc");
require_once("captcha.inc");

function urlsafe_b64encode($string)
{
    $data = base64_encode($string);
    $data = str_replace(array('+','/','='),array('-','_','.'),$data);
    return $data;
}

function urlsafe_b64decode($string)
{
    $data = str_replace(array('-','_','.'),array('+','/','='),$string);
    $mod4 = strlen($data) % 4;
    if ($mod4) $data .= substr('====', $mod4);
    return base64_decode($data);
}

function Open_Captcha_DB()
{
    global $captcha_conf;
    $captcha_db = dba_open($captcha_conf['captcha_dir'] . "/captcha.db", "r", $captcha_conf['captcha_dba_driver']);
    if ( $captcha_db === False )
      die("Failed to open captcha DB. The library is not correctly configured.");
    return $captcha_db;
}

// Creates a new captcha challenge and returns it in an array
function Random_Captcha()
{
    $captcha_db = Open_Captcha_DB();
    $total = dba_fetch("total", $captcha_db);
    assert( $total > 0 ); // Database probably corrupted.
    $idx = rand(0, $total-1);
    $answ = dba_fetch("answ" . $idx, $captcha_db);
    assert( $answ !== False ); // Database probably corrupted.

    $captcha["answer"] = '' . $answ;
    $captcha["time"] = time();
    return $captcha;
}

// Returns captcha image data for given answer. Use -1 for "invalid.gif".
function Get_Captcha_Pic( $answ )
{
    $captcha_db = Open_Captcha_DB();
    $file = dba_fetch("" . (int)$answ, $captcha_db);
    assert( $file !== False ); // Database probably corrupted.
    return $file;
}

// Encrypts given string with the secret $crypto_key
function Encrypt_Str( $str )
{
    global $captcha_conf;
    $salt = "" . rand(1000, 9999);
    $td = mcrypt_module_open('tripledes', '', 'ecb', '');
    $iv = mcrypt_create_iv (mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
    $crypto_key = substr($captcha_conf['crypto_key'], 0,mcrypt_enc_get_key_size($td));
    mcrypt_generic_init($td, $crypto_key, $iv);
    $encrypted_data = mcrypt_generic($td, $str . $salt);
    mcrypt_generic_deinit($td);
    mcrypt_module_close($td);
    return urlsafe_b64encode($encrypted_data);
}

// Decrypts given string with the secret $crypto_key
function Decrypt_Str( $str )
{
    global $captcha_conf;
    $td = mcrypt_module_open('tripledes', '', 'ecb', '');
    $iv = mcrypt_create_iv (mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
    $crypto_key = substr($captcha_conf['crypto_key'], 0,mcrypt_enc_get_key_size($td));
    mcrypt_generic_init($td, $crypto_key, $iv);
    $str = urlsafe_b64decode($str);
    if ( !is_string($str))
        return False;
    $plain_data = mdecrypt_generic($td, $str);
    if ( !is_string($plain_data))
        return False;
    $plain_data = rtrim($plain_data, "\0");
    $plain_data = substr($plain_data, 0, strlen($plain_data)-4); // remove salt
    mcrypt_generic_deinit($td);
    mcrypt_module_close($td);
    return $plain_data;
}

// Serializes given captcha array to string and encrypts it.
function Pack_Answer( $captcha )
{
    return Encrypt_Str("valid|" .  $captcha["answer"] . "|" . $captcha["time"]);
}

// Decrypts given captcha string and deserializes it into an array.
function Unpack_Answer( $packed_captcha )
{
    $str = Decrypt_Str( $packed_captcha );
    $res = explode ( "|", $str, 4 );
    if ( !isset($res[0]) || $res[0] != "valid" )
        return false;
    $res = Array("answer" => $res[1], "time" => $res[2]);
    return $res;
}

// Creates a new captcha challenge and prints it to default output.
function Print_New_Captcha()
{
  global $captcha_conf;
  $question_text = $captcha_conf['question_text'];
  $wrong_text = $captcha_conf['wrong_text'];
  $checking_text = $captcha_conf['checking_text'];
  $change_text = $captcha_conf['change_text'];
  $captcha_url = $captcha_conf['captcha_url'];
   
  $captcha = Random_Captcha();
  $packed = Pack_Answer( $captcha );
?>
  <script type="text/javascript">
  <!--
    var xmlhttp=false;
    /*@cc_on @*/
    /*@if (@_jscript_version >= 5)
    // JScript gives us Conditional compilation, we can cope with old IE versions.
    // and security blocked creation of the objects.
     try { xmlhttp = new ActiveXObject("Msxml2.XMLHTTP"); } catch (e) {
      try { xmlhttp = new ActiveXObject("Microsoft.XMLHTTP"); } catch (E) { xmlhttp = false; }
     }
    @end @*/
    if (!xmlhttp && typeof XMLHttpRequest!='undefined') {
            try { xmlhttp = new XMLHttpRequest(); } catch (e) { xmlhttp=false; }
    }
    if (!xmlhttp && window.createRequest) {
            try { xmlhttp = window.createRequest(); } catch (e) { xmlhttp=false; }
    }

    // An onKeyPress handler: limit input to numbers and send an
    // asynchronous captcha test request when the field is full.
    function captcha_keytest(myfield, e)
    {
      var key;
      var keychar;
      if (window.event)
         key = window.event.keyCode;
      else if (e)
         key = e.which;
      else
         return true;
      keychar = String.fromCharCode(key);

      if ( myfield.checked_ok == 1 )
        return false;

      // Allow control keys & numbers
      if ((key==null) || ((key<=57 || key==127) && key != 32))
      {
        // Initiate captcha test
        setTimeout(function() {
            if ( myfield.value.length == 6)
            {
              if ( myfield.checked_ok == 1 || myfield.checking == 1 )
                return;
              myfield.checking = 1;
              document.getElementById('captcha-status').innerHTML = "<strong style='color: gray;'><?php print $checking_text; ?>";
              xmlhttp.open("GET", "<?php print "$captcha_url/";  ?>captcha.php?captcha-answ=" + myfield.value + "&captcha-packed=" + escape(document.getElementById('captcha-packed').value), true);
              xmlhttp.onreadystatechange=function() {
                if (xmlhttp.readyState==4) {
                  if ( xmlhttp.responseText == "OK" ) {
                    document.getElementById('captcha-status').innerHTML = "<strong style='color: green;'>OK</strong>";
                    myfield.style.background='#80FF80';
                    myfield.checked_ok = 1;
                  } else {
                    var reason = "";
                    if ( xmlhttp.responseText.length > 1 || xmlhttp.responseText[0] != ' ' )
                      reason = ": " + xmlhttp.responseText;
                    document.getElementById('captcha-status').innerHTML = "<strong style='color: red;'><?php print $wrong_text; ?>" + reason + "</strong>";
                  }
                  myfield.checking = 0;
                }
              }
              xmlhttp.send(null);
            }}, 250);
         return true;
      }
      else
      {
         // Illegal key: flash the field red for 1/2 second.
         myfield.style.background='#FF8080';
         setTimeout(function() { myfield.style.background='white'; }, 500);
         return false;
      }
    }

    function change_captcha()
    {
        xmlhttp.open("GET", "<?php print "$captcha_url/captcha.php";  ?>", true);
        xmlhttp.onreadystatechange = function() {
          if (xmlhttp.readyState==4) document.getElementById('captcha-area').innerHTML = xmlhttp.responseText; }
          xmlhttp.send(null);
        return false;
    }

    setTimeout( function() {
      document.getElementById('captcha-status').innerHTML =
        "<a href='#' style='font-size: 90%;' onClick='change_captcha()'><?php print $change_text; ?></a>";
      }, 500);
  //-->
  </script>

  <div id='captcha-area'>
    <img src='<?php print "$captcha_url/captcha.php?img=" . $packed; ?>' alt='CAPTCHA' width='200' height='100' style='border: 1px solid black;'><br/>
    <?php print $question_text; ?><br/>
    <input type='text' id='captcha-input' name='captcha-answ' size='6' maxlength='6' onKeyPress="return captcha_keytest(this, event)">
    <input type='hidden' name='captcha-packed' id='captcha-packed' value='<?php print htmlspecialchars($packed); ?>'>
    <span id='captcha-status'></span>
  </div>
<?php
}

// Checks user's answer to a captcha. Returns 'true' if the answer was
// correct and an error string (possibly empty) otherwise.
// If '$answ' is given (not False), the function uses $answ['answ'] and
// $answ['packed'] for checking, otherwise $_REQUEST['answ'] and $_REQUEST['packed'];
function Check_Captcha_Answer( $answ = False )
{
  global $captcha_conf;
  $captcha_timeout = $captcha_conf['captcha_timeout'];
  if ( !is_array($answ))
    $answ = $_REQUEST;

  if ( !isset($answ['captcha-packed']))
    return "missing parameter 'captcha-packed'";
  if ( !isset($answ['captcha-answ']))
    return "missing parameter 'captcha-answ'";

  $unp = Unpack_Answer($answ['captcha-packed']);
  if ( !is_array($unp))
    $unp = Unpack_Answer(urldecode($answ['captcha-packed']));

  if ( time() - (int)$unp['time'] > $captcha_timeout || time() - (int)$unp['time'] < 0 )
  {
    sleep(2);
    return "challenge expired";
  }
  else
  {
    if ( is_array($unp) && (int)$unp['answer'] == (int)$answ['captcha-answ'] &&
         time() - (int)$unp['time'] < $captcha_timeout )
      return True;
  }
  sleep(2);
  return " "; // failure
}

?>